import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_background_geolocation_example/advanced/util/dialog.dart' as util;

import 'package:flutter_background_geolocation/flutter_background_geolocation.dart' as bg;

import './config/env.dart';

class RegistrationView extends StatefulWidget {
  @override
  State createState() => _RegistrationViewState();
}

class _RegistrationViewState extends State<RegistrationView> {
  static const USERNAME_REGEXP = r"^[a-zA-Z0-9_-]*$";

  String _deviceId;
  bg.DeviceInfo _deviceInfo;

  String _orgname;
  String _orgnameRaw;

  String _username;
  String _usernameRaw;

  final _formKey = GlobalKey<FormState>();

  final _orgnameController = TextEditingController();
  final _usernameController = TextEditingController();

  void initState() {
    super.initState();

    setState(() {
      _orgname = 'orgname';
      _username = 'username';
    });

    bg.DeviceInfo.getInstance().then((bg.DeviceInfo info) {
      setState(() {
        _deviceId = "${info.manufacturer} ${info.model}";
        _deviceInfo = info;
      });
    });
  }

  void _onClickClose() {
    bg.BackgroundGeolocation.playSound(util.Dialog.getSoundId("CLOSE"));
    Navigator.of(context).pop();
  }

  void _onClickSave() {
    bg.BackgroundGeolocation.playSound(util.Dialog.getSoundId("CLOSE"));
    //Navigator.of(context).pop();
    if (_formKey.currentState.validate()) {
      // If the form is valid, display a Snackbar.
      print("*********** VALIDATION SUCCESS");
    } else {
      print("*********** VALIDATION FAIL");
    }
  }

  bool _usernameIsValid(String username) {
    return (username != null) && new RegExp(USERNAME_REGEXP).hasMatch(username) && (username.length > 0);
  }

  @override
  Widget build(BuildContext context) {
    if (_orgname == null || _deviceInfo == null) {
      return new Scaffold(
          body: new Text('Loading...')
      );
    }

    final re = new RegExp(r"\s+\b|\b\s");
    _orgnameRaw = _orgname;
    _usernameRaw = _username;

    //TextEditingController _usernameController = new TextEditingController(text: _usernameRaw);
    //TextEditingController _orgnameController = new TextEditingController(text: _orgnameRaw);
    _orgnameController.value = TextEditingValue(text: _orgnameRaw, selection: TextSelection.collapsed(offset: _orgnameRaw.length));
    _usernameController.value = TextEditingValue(text: _usernameRaw, selection: TextSelection.collapsed(offset: _usernameRaw.length));
    return new Scaffold(
        appBar: new AppBar(
            brightness: Brightness.light,
            leading: IconButton(onPressed: _onClickClose, icon: Icon(Icons.close), color: Colors.black),
            title: const Text('Device Registration', textAlign: TextAlign.right),
            backgroundColor: Colors.amberAccent,
            iconTheme: IconThemeData(color: Colors.black),
            actions: <Widget>[
              MaterialButton(
                onPressed: _onClickSave,
                child: Text("REGISTER"),
                height: 50.0)
            ]
        ),
        body: Form(
          key: _formKey,
          child: Container(
            padding: EdgeInsets.all(10.0),
            child: Column(
              children: <Widget>[
                Text("Please provide a unique Organization and User identifier to register your device with the Demo Server"),
                const Text(""),
                Text("$_deviceId", textAlign: TextAlign.center, style: TextStyle(fontStyle: FontStyle.italic, fontWeight: FontWeight.bold)),
                TextFormField(
                  controller: _orgnameController,
                  validator: (value) {
                    if (!_usernameIsValid(value)) {
                      return 'Invalid organization name.';
                    } else {
                      return null;
                    }
                  },
                  //autofocus: true,
                  decoration: InputDecoration(
                      labelText: 'Organization name', hintText: 'eg. Company name'),
                ),
                TextFormField(
                  controller: _usernameController,
                  validator: (value) {
                    if (!_usernameIsValid(value)) {
                      return 'Invalid username.';
                    } else {
                      return null;
                    }
                  },
                  autofocus: false,
                  decoration: InputDecoration(
                      labelText: 'Username', hintText: 'eg. Github username or initials'),
                ),
              ],
            ),
          ),
        )
    );
  }
}